KnotVec

Struct KnotVec 

pub struct KnotVec<N: ScalarT> { /* private fields */ }
Vector of knots in non-decreasing order.

Knot values exist in the parameter space of a NURBS curve. They partition the total 1D parameter range into regions over which the interpolating polynomials of the NURBS curve are active. Thus, in combination with the degree of a NURBS curve, they define the non-uniform B-spline basis functions.

Implementations

Source

impl<N: ScalarT> KnotVec<N>

pub fn new(knots: Vec<N>) -> Option<Self>

Creates a new knot vector if possible.

A new knot vector must satisfy the following criteria:

  • it must contain >= 2 elements
  • it must be sorted (non-decreasing)
  • it must represent a non-zero span (the last knot cannot be equal to the first)
Parameters
  • knots - a vector of knot values in non-decreasing order
Example
let knots = KnotVec::new(vec![0.0, 0.0, 0.0, 1.0, 1.0, 1.0]).unwrap();
pub fn len(&self) -> usize

Returns the number of knots in the knot vector.

Example
let knots = KnotVec::new(vec![0.0, 0.0, 1.0, 1.0]).unwrap();
assert_eq!(knots.len(), 4);
pub fn is_clamped(&self, degree: usize) -> bool

Checks if a knot vector is clamped.

A knot vector is clamped if the first knot value is repeated degree + 1 times at the start of the knot vector (ie. its multiplicity is degree + 1), and if the last knot is repeated degree + 1 times at the end of the knot vector.

Parameters
  • degree - degree of the NURBS curve
pub fn is_empty(&self) -> bool

Checks if the knot vector is empty (always returns false).

pub fn min_u(&self) -> N

Returns the minimum parameter value contained in this knot vector.

pub fn max_u(&self) -> N

Returns the maximum parameter value contained in this knot vector.

pub fn clamp(&self, u: N) -> N

Clamp a parameter value to the allowed range of the parameter.

Parameters
  • u - the parameter value to clamp in the range min_u <= u <= max_u
pub fn find_span(&self, u: N) -> usize

Finds the index of the span inside the knot vector which contains the parameter value u.

Each pair of knots in the knot vector defines a span. Spans are zero length for knots with a multiplicity > 1. Given a parameter value, u, this function returns the index of the knot span which contains u.

The knot span index, i = knots.find_span(u), which contains u, satisfies the relationship that:

knots[i] <= u < knots[i+1], when u < knots.max_u()
knots[i] < u == knots[i+1], when u == knots.max_u()
Parameters
  • u - the parameter value for which to find the span
Examples
let knots = KnotVec::new(vec![0.0, 0.0, 0.5, 1.0, 1.0]).unwrap();
assert_eq!(knots.find_span(0.0), 1);
assert_eq!(knots.find_span(0.6), 2);
assert_eq!(knots.find_span(1.0), 2); // note u=knots.max_u() is a bit special
Panics

Panics if the parameter u is outside the allowed range of the knot vector.